const GdkPixbuf = imports.gi.GdkPixbuf;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const GObject = imports.gi.GObject;
const Gtk = imports.gi.Gtk;
const St = imports.gi.St;
const Lang = imports.lang;

const Gettext = imports.gettext.domain('nls1729-extensions');
const _ = Gettext.gettext;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const _N = function(x) { return x; }
const RMV_ACTIV = _N("Remove the Activities Button.");
const TXT_INSTS = _N("To change text enter new text then press Apply.");
const HIDE_TEXT = _N("Hide text.");
const ICO_INSTS = _N("To change icon press Select then choose icon.");
const HIDE_ICON = _N("Hide icon.");
const NADA_HOTC = _N("Disable upper left hot corner.");
const RST_INSTS = _N("To reset to extension defaults press Reset.");
const TITLE = _N("Choose Icon");
const HIDE_NOTE = _N("Removing the Activities Button disables all Activities functions. Use left Super key to access Overview.");
const DEFAULT_ICO = ExtensionUtils.getCurrentExtension().path + '/face-smile-3.svg'; // From Tango Project - Public Domain
const ACTION = 0;
const GTK_RESPONSE_CANCEL = -6;
const GTK_RESPONSE_ACCEPT = -3;
const GTK_STOCK_CANCEL = 'gtk-cancel';
const GTK_STOCK_OPEN = 'gtk-open';
const REMOVED_KEY = 'activities-config-button-removed';
const NEW_ICO_KEY = 'activities-config-button-icon-path';
const ORI_TXT_KEY = 'original-activities-button-text';
const NEW_TXT_KEY = 'activities-config-button-text';
const NO_HOTC_KEY = 'activities-config-hot-corner';
const NO_ICON_KEY = 'activities-config-button-no-icon';
const NO_TEXT_KEY = 'activities-config-button-no-text';

function init() {
    Convenience.initTranslations();
}

const ActivitiesConfiguratorSettingsWidget = new GObject.Class({
    Name: 'ActivitiesConfigurator.Prefs.ActivitiesConfiguratorSettingsWidget',
    GTypeName: 'ActivitiesConfiguratorSettingsWidget',
    Extends: Gtk.Grid,

    _init: function(params) {	
        this.parent(params);
        this.margin = this.row_spacing = this.column_spacing = 6;
	this._settings = Convenience.getSettings();
        this.attach(new Gtk.Label({ label: _(TXT_INSTS), wrap: true, xalign: 0.0 }), 1, 0, 2, 1);
        this.attach(new Gtk.Label({ label: _(HIDE_TEXT), wrap: true, xalign: 0.0 }), 2, 2, 1, 1);
        this.attach(new Gtk.Label({ label: _(ICO_INSTS), wrap: true, xalign: 0.0 }), 2, 3, 1, 1);
        this.attach(new Gtk.Label({ label: _(HIDE_ICON), wrap: true, xalign: 0.0 }), 2, 4, 1, 1);
        this.attach(new Gtk.Label({ label: _(NADA_HOTC), wrap: true, xalign: 0.0 }), 2, 5, 1, 1);
        this.attach(new Gtk.Label({ label: _(RMV_ACTIV), wrap: true, xalign: 0.0 }), 2, 6, 1, 1);
        this.attach(new Gtk.Label({ label: _(HIDE_NOTE), wrap: true, xalign: 0.0 }), 1, 7, 2, 1);
        this.attach(new Gtk.Label({ label: _(RST_INSTS), wrap: true, xalign: 0.0 }), 2, 9, 1, 1);
        let applyBtn = new Gtk.Button({ label: _("Apply ") });
        this.attach(applyBtn, 1, 1, 1, 1);
        this._entry = new Gtk.Entry({ hexpand: true });
        this.attach(this._entry, 2, 1, 1, 1);
        applyBtn.connect('clicked', Lang.bind(this, this._setActivitiesText));
        this._noText = new Gtk.Switch({active: this._settings.get_boolean(NO_TEXT_KEY)});
        this.attach(this._noText, 1, 2, 1, 1);
        this._noText.connect('notify::active', Lang.bind(this, this._setNoText));
        this._iconImage = new Gtk.Image();
        this._iconPath = this._settings.get_string(NEW_ICO_KEY) || DEFAULT_ICO;
        this._loadIcon(this._iconPath);
        this.attach(this._iconImage, 0, 3, 1, 1);
        let iconBtn = new Gtk.Button({ label: _("Select") });
        this.attach(iconBtn, 1, 3, 1, 1);
        iconBtn.connect('clicked', Lang.bind(this, this._setActivitiesIcon));
        this._noIcon = new Gtk.Switch({active: this._settings.get_boolean(NO_ICON_KEY)});
        this.attach(this._noIcon, 1, 4, 1, 1);
        this._noIcon.connect('notify::active', Lang.bind(this, this._setNoIcon));
        this._noHotCorner = new Gtk.Switch({active: this._settings.get_boolean(NO_HOTC_KEY)});
        this.attach(this._noHotCorner, 1, 5, 1, 1);
        this._noHotCorner.connect('notify::active', Lang.bind(this, this._setNoHotCorner));
        this._noActivities = new Gtk.Switch({active: this._settings.get_boolean(REMOVED_KEY)});
        this.attach(this._noActivities, 1, 6, 1, 1);
        this._noActivities.connect('notify::active', Lang.bind(this, this._setNoActivities));
        let horzSeparator = new Gtk.HSeparator();
        this.attach(horzSeparator, 0, 8, 3, 1);
        let defaultsBtn = new Gtk.Button({ label: _("Reset ") } );
        this.attach(defaultsBtn, 1, 9, 1, 1);
        defaultsBtn.connect('clicked', Lang.bind(this, this._resetSettings));
    },

    _setActivitiesText: function() {
        let text = this._entry.get_text();
        if(text != '') {
            this._settings.set_string(NEW_TXT_KEY, text);
            this._entry.set_text('');
        }
    },

    _setNoText: function(object) {
        this._settings.set_boolean(NO_TEXT_KEY, object.active);
    },

    _setActivitiesIcon: function() {
        let dialog = new Gtk.FileChooserDialog({ title: _(TITLE), action: ACTION });
        dialog.add_button(GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
        dialog.add_button(GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT);
        dialog.set_filename(this._iconPath);
        let filter = new Gtk.FileFilter();
        filter.set_name(_("Images"));
        filter.add_pattern("*.png");
        filter.add_pattern("*.jpg");
        filter.add_pattern("*.gif");
        filter.add_pattern("*.svg");
        filter.add_pattern("*.ico");
        dialog.add_filter(filter);
        let response = dialog.run();
        if(response == GTK_RESPONSE_ACCEPT) {
            let filename = dialog.get_filename()
            if(filename != this._iconPath) {
                this._iconPath = filename;
                this._loadIcon(filename);
                this._settings.set_string(NEW_ICO_KEY, filename);
            }
        }
        dialog.destroy();
    },

    _setNoIcon: function(object) {
        this._settings.set_boolean(NO_ICON_KEY, object.active);
    },

    _setNoHotCorner: function(object) {
        this._settings.set_boolean(NO_HOTC_KEY, object.active);
    },

    _setNoActivities: function(object) {
        this._settings.set_boolean(REMOVED_KEY, object.active);
    },

    _resetSettings: function() {
        this._noActivities.set_active(false);
        let default_txt = this._settings.get_string(ORI_TXT_KEY);
        this._settings.set_string(NEW_TXT_KEY, default_txt);
        this._noText.set_active(false);
        this._settings.set_string(NEW_ICO_KEY, DEFAULT_ICO);
        this._iconPath = DEFAULT_ICO;
        this._loadIcon(this._iconPath);
        this._noIcon.set_active(false);
        this._noHotCorner.set_active(false);
    },

    _loadIcon: function(path) {
        let pixbuf = new GdkPixbuf.Pixbuf.new_from_file_at_scale(path, 32, 32, null);
        this._iconImage.set_from_pixbuf(pixbuf);
    }
});

function buildPrefsWidget() {
    let widget = new ActivitiesConfiguratorSettingsWidget();
    widget.show_all();
    return widget;
}
